<!-- Favicon thumbnail - small favicon preview -->
<div class="extractor-thumbnail favicon-thumbnail" style="width: 100%; height: 100px; display: flex; align-items: center; justify-content: center; background: #fff;">
    {% if output_path %}
        <img src="{{ output_path }}"
             alt="Favicon"
             style="width: 30px; height: 30px; max-width: 30px; max-height: 30px; object-fit: contain;"
             loading="lazy">
    {% endif %}
</div>
