<!-- Claude Code Cleanup fullscreen - show cleanup report -->
<iframe class="full-page-iframe"
        src="{{ output_path }}"
        name="preview"
        sandbox="allow-top-navigation-by-user-activation">
</iframe>
