<span class="abx-output-icon abx-output-icon--trafilatura" title="Trafilatura"><svg width="16" height="16" viewBox="0 0 24 24" aria-hidden="true" focusable="false" fill="none" stroke="currentColor" stroke-width="1.8" stroke-linecap="round" stroke-linejoin="round"><rect x="3" y="5" width="18" height="14" rx="2"/><path d="M7 9h10"/><path d="M8.5 12h7"/><path d="M10.5 15h3"/><path d="M12 15v3"/></svg></span>
