<span class="abx-output-icon abx-output-icon--search_backend_ripgrep" title="Search"><svg width="16" height="16" viewBox="0 0 24 24" aria-hidden="true" focusable="false" fill="none" stroke="currentColor" stroke-width="1.8" stroke-linecap="round" stroke-linejoin="round"><circle cx="11" cy="11" r="5"/><path d="M16 16l4 4"/></svg></span>
