<span class="abx-output-icon abx-output-icon--ublock" title="uBlock"><svg width="16" height="16" viewBox="0 0 24 24" aria-hidden="true" focusable="false" fill="none" stroke="currentColor" stroke-width="1.8" stroke-linecap="round" stroke-linejoin="round"><path d="M12 3l6 2.5v5.5c0 4.1-2.5 7.6-6 10-3.5-2.4-6-5.9-6-10V5.5z"/><path d="M8 12h8"/><path d="M9.5 9.5l5 5"/></svg></span>
