<span class="abx-output-icon abx-output-icon--brew" title="Homebrew"><svg width="16" height="16" viewBox="0 0 24 24" aria-hidden="true" focusable="false" fill="none" stroke="currentColor" stroke-width="1.8" stroke-linecap="round" stroke-linejoin="round"><path d="M7 8h8a1 1 0 0 1 1 1v5a4 4 0 0 1-4 4h-2a4 4 0 0 1-4-4V9a1 1 0 0 1 1-1z"/><path d="M16 10h1.5a2.5 2.5 0 0 1 0 5H16"/><path d="M9 4v2"/><path d="M12 3.5V6"/><path d="M15 4v2"/></svg></span>
