<!-- Singlefile thumbnail - scaled down iframe preview of archived HTML -->
<div class="extractor-thumbnail singlefile-thumbnail" style="width: 100%; height: 100px; overflow: hidden; background: #fff;">
    <iframe src="{{ output_path }}"
            style="width: 400%; height: 400px; transform: scale(0.25); transform-origin: top left; pointer-events: none; border: none;"
            loading="lazy"
            sandbox="allow-same-origin">
    </iframe>
</div>
