<span class="abx-output-icon abx-output-icon--istilldontcareaboutcookies" title="Cookie Banners"><svg width="16" height="16" viewBox="0 0 24 24" aria-hidden="true" focusable="false" fill="none" stroke="currentColor" stroke-width="1.8" stroke-linecap="round" stroke-linejoin="round"><path d="M12 5.5a6.5 6.5 0 1 0 6.5 6.5c-2.2 0-4-1.8-4-4 0-.9.3-1.8.8-2.5A6.4 6.4 0 0 0 12 5.5z"/><circle cx="9.5" cy="10" r="1" fill="currentColor" stroke="none"/><circle cx="11.5" cy="14" r="1" fill="currentColor" stroke="none"/><path d="M5 19L19 5"/></svg></span>
