<span class="abx-output-icon abx-output-icon--puppeteer" title="Puppeteer"><svg width="16" height="16" viewBox="0 0 24 24" aria-hidden="true" focusable="false" fill="none" stroke="currentColor" stroke-width="1.8" stroke-linecap="round" stroke-linejoin="round"><path d="M8 4h8"/><path d="M12 4v2"/><path d="M8 4l2 3"/><path d="M16 4l-2 3"/><rect x="4" y="6" width="16" height="12" rx="2"/><path d="M4 10h16"/></svg></span>
